"""MGZ database API."""

import csv
import logging
import multiprocessing
import os
import sys
import tempfile
import zipfile

import iso8601
import requests_cache
from sqlalchemy.exc import IntegrityError

import voobly
from mgzdb import queries
from mgzdb.add import add_file
from mgzdb.compress import decompress
from mgzdb.schema import get_session, reset, Tag, File, Match, Series
from mgzdb.util import get_store, fetch_file


LOGGER = logging.getLogger(__name__)
SOURCE_VOOBLY = 'voobly'
SOURCE_ZIP = 'zip'
SOURCE_CLI = 'cli'
SOURCE_CSV = 'csv'
QUERY_MATCH = 'match'
QUERY_FILE = 'file'
QUERY_SERIES = 'series'
QUERY_SUMMARY = 'summary'


class API: # pylint: disable=too-many-instance-attributes
    """MGZ Database API."""

    def __init__(self, db_path, store_host, store_path, consecutive=False, callback=None, **kwargs):
        """Initialize sessions."""

        self.session = get_session(db_path)
        self.process_args = (store_host, store_path)
        self.temp_dir = tempfile.TemporaryDirectory()
        self.store = None
        self.store_host = store_host
        self.store_path = store_path
        self.db_path = db_path
        self.callback = callback
        self.consecutive = consecutive
        self.voobly = voobly.get_session(
            key=kwargs.get('voobly_key'),
            username=kwargs.get('voobly_username'),
            password=kwargs.get('voobly_password')
        )
        self.voobly_key = kwargs.get('voobly_key')
        self.voobly_username = kwargs.get('voobly_username')
        self.voobly_password = kwargs.get('voobly_password')
        self.pool = None
        self.total = 0
        LOGGER.info("connected to database @ %s", db_path)

    def start(self):
        """Start child processes."""
        def init_worker(function):
            function.connections = {
                'store': get_store(self.store_host),
                'session': get_session(self.db_path),
                'aoe2map': requests_cache.CachedSession(backend='memory'),
                'voobly': voobly.get_session(
                    key=self.voobly_key,
                    username=self.voobly_username,
                    password=self.voobly_password
                )
            }
        self.pool = multiprocessing.Pool(
            initializer=init_worker,
            initargs=(add_file, ),
            processes=1 if self.consecutive else None
        )

    def finished(self):
        """Wait for child processes to end."""
        try:
            self.pool.close()
            self.pool.join()
        except KeyboardInterrupt:
            print('user requested exit')
            sys.exit()
        finally:
            self.temp_dir.cleanup()

    def add_file(self, *args, **kwargs):
        """Add file via process pool."""
        if not self.pool:
            raise ValueError('call start() first')
        self.total += 1
        self.pool.apply_async(
            add_file,
            args=(*self.process_args, *args),
            kwds=kwargs,
            callback=self._file_added,
            error_callback=self._critical_error
        )

    def add_url(self, voobly_url, download_path, tags, force=False):
        """Add a match via Voobly url."""
        voobly_id = voobly_url.split('/')[-1]
        try:
            match = voobly.get_match(self.voobly, voobly_id)
        except voobly.VooblyError as error:
            LOGGER.error("failed to get match: %s", error)
            return
        for player in match['players']:
            filename = voobly.download_rec(self.voobly, player['url'], download_path)
            self.add_file(
                os.path.join(download_path, filename),
                SOURCE_VOOBLY,
                voobly_url,
                tags,
                voobly_id=voobly_id,
                played=match['timestamp'],
                force=force,
                user_data=player
            )

    def add_series(self, zip_path, tags, series=None, challonge_id=None, force=False):
        """Add a series via zip file."""
        with zipfile.ZipFile(zip_path) as series_zip:
            LOGGER.info("[%s] opened archive", os.path.basename(zip_path))
            series_zip.extractall(self.temp_dir.name)
            for filename in sorted(series_zip.namelist()):
                if filename.endswith('/'):
                    continue
                LOGGER.info("[%s] processing member file %s", os.path.basename(zip_path), filename)
                self.add_file(
                    os.path.join(self.temp_dir.name, filename),
                    SOURCE_ZIP,
                    os.path.basename(zip_path),
                    tags,
                    series,
                    challonge_id,
                    force=force
                )
            LOGGER.info("[%s] finished", os.path.basename(zip_path))

    def add_csv(self, csv_path, download_path, tags, force=False):
        """Add matches from CSV."""
        LOGGER.info("opening %s for import", csv_path)
        with open(csv_path) as csv_file:
            LOGGER.info("[%s] starting parse", os.path.basename(csv_path))
            for num, row in enumerate(csv.DictReader(csv_file)):
                LOGGER.info("[%s] adding data from row %d", os.path.basename(csv_path), num + 1)
                url = row['PlayerRecording'].replace(voobly.BASE_URL, '')
                filename = voobly.download_rec(self.voobly, url, download_path)
                old_path = os.path.join(download_path, filename)
                new_path = os.path.join(download_path, '{}.{}'.format(row['PlayerId'], filename))
                os.rename(old_path, new_path)
                self.add_file(
                    new_path,
                    SOURCE_CSV,
                    os.path.basename(csv_path),
                    tags,
                    voobly_id=row['MatchId'],
                    played=iso8601.parse_date(row['MatchDate']),
                    force=force,
                    user_data={
                        'id': row['PlayerId'],
                        'username': row['PlayerName'],
                        'clan': None,
                        'color_id': None, # this won't work until we can set `color_id`
                        'rate_before': row['PlayerPreRating'],
                        'rate_after': row['PlayerPostRating']
                    }
                )
            LOGGER.info("[%s] finished parse", os.path.basename(csv_path))

    def remove(self, file_id=None, match_id=None, series_id=None):
        """Remove a file, match, or series."""
        if file_id:
            obj = self.session.query(File).get(file_id)
            if len(obj.match.files) == 1:
                obj = obj.match.files[0].match
        if match_id:
            obj = self.session.query(Match).get(match_id)
        if series_id:
            obj = self.session.query(Series).get(series_id)
        if obj:
            self.session.delete(obj)
            self.session.commit()
        else:
            print('not found')

    def tag(self, match_id, tags):
        """Tag a match."""
        match = self.session.query(Match).get(match_id)
        for tag in tags:
            try:
                obj = Tag(name=tag, match=match)
                self.session.add(obj)
                self.session.commit()
            except IntegrityError:
                self.session.rollback()

    def get(self, file_id):
        """Get a file from the store."""
        if not self.store:
            self.store = get_store(self.store_host)
        mgz_file = self.session.query(File).get(file_id)
        store_path = os.path.join(self.store_path, mgz_file.filename)
        return mgz_file.original_filename, decompress(fetch_file(self.store, store_path))

    def reset(self):
        """Reset database."""
        reset(self.db_path)
        LOGGER.info("reset database")

    def query(self, query_type, **kwargs):
        """Query database."""
        if query_type == QUERY_MATCH:
            return queries.get_match(self.session, kwargs['match_id'])
        if query_type == QUERY_FILE:
            return queries.get_file(self.session, kwargs['file_id'])
        if query_type == QUERY_SERIES:
            return queries.get_series(self.session, kwargs['series_id'])
        if query_type == QUERY_SUMMARY:
            return queries.get_summary(self.session)
        raise ValueError('unsupported query type')

    def _file_added(self, success): # pylint: disable=unused-argument
        """Callback when file is addded."""
        if self.callback:
            self.callback()

    def _critical_error(self, val): # pylint: disable=unused-argument
        """Handle critical errors from child processes."""
        if self.callback:
            self.callback()



"""MGZ database API."""

import hashlib
import io
import logging
import os
import sys
from datetime import timedelta

import pkg_resources
from sqlalchemy.orm.exc import NoResultFound
from sqlalchemy.exc import IntegrityError

import mgz.const
import mgz.summary
import voobly

from mgzdb.schema import (
    Match, VooblyUser, Tag, Series, File, Source, Mod, VooblyLadder,
    Player, Civilization, Map, VooblyClan, Team
)
from mgzdb.util import copy_file, parse_filename_timestamp
from mgzdb.compress import compress


LOGGER = logging.getLogger(__name__)
LOG_ID_LENGTH = 8
COMPRESSED_EXT = '.mgc'
MAP_URL = 'https://aoe2map.net/api/rms/file'

def add_file(
        store_host, store_path, rec_path, source, reference, tags,
        series=None, challonge_id=None, voobly_id=None, played=None,
        force=False, user_data=None
    ):
    """Wrapper around AddFile class for parallelization."""
    obj = AddFile(
        add_file.connections, store_host, store_path # pylint: disable=no-member
    )
    try:
        return obj.add_file(
            rec_path, source, reference, tags, series=series, challonge_id=challonge_id,
            voobly_id=voobly_id, played=played, force=force, user_data=user_data
        )
    except KeyboardInterrupt:
        sys.exit()


class AddFile:
    """Add file to MGZ Database."""

    def __init__(self, connections, store_host, store_path):
        """Initialize sessions."""
        self.store_host = store_host
        self.store_path = store_path
        self.store = connections['store']
        self.session = connections['session']
        self.voobly = connections['voobly']
        self.aoe2map = connections['aoe2map']

    def add_file(
            self, rec_path, source, reference, tags, series=None, challonge_id=None,
            voobly_id=None, played=None, force=False, user_data=None
    ):
        """Add a single mgz file."""

        if not os.path.isfile(rec_path):
            LOGGER.error("%s is not a file", rec_path)
            return False

        original_filename = os.path.basename(rec_path)

        with open(rec_path, 'rb') as handle:
            data = handle.read()

        file_hash = hashlib.sha1(data).hexdigest()
        log_id = file_hash[:LOG_ID_LENGTH]
        LOGGER.info("[f:%s] add started", log_id)

        if self.session.query(File).filter_by(hash=file_hash).count() > 0:
            LOGGER.warning("[f:%s] file already exists", log_id)
            return False

        try:
            handle = io.BytesIO(data)
            summary = mgz.summary.Summary(handle, len(data))
        except RuntimeError:
            LOGGER.error("[f:%s] invalid mgz file", log_id)
            return False

        compressed_filename = '{}{}'.format(file_hash, COMPRESSED_EXT)
        new_path = os.path.join(self.store_path, compressed_filename)
        copy_file(io.BytesIO(compress(io.BytesIO(data))), self.store, new_path)
        LOGGER.info("[f:%s] copied to %s:%s", log_id, self.store_host, new_path)

        match_hash = summary.get_hash().hexdigest()
        try:
            match = self.session.query(Match).filter_by(hash=match_hash).one()
            LOGGER.info("[f:%s] match already exists; appending", log_id)
        except NoResultFound:
            LOGGER.info("[f:%s] adding match", log_id)
            if not played:
                played = parse_filename_timestamp(original_filename)
            match = self._add_match(summary, played, tags, match_hash, series, challonge_id, voobly_id, force)
            if not match:
                return False

        self._update_match_user(match.id, user_data)

        new_file = self._get_unique(
            File, ['hash'],
            filename=compressed_filename,
            original_filename=original_filename,
            hash=file_hash,
            size=summary.size,
            reference=reference,
            match=match,
            source=self._get_unique(Source, name=source),
            owner_number=summary.get_owner(),
            parser_version=pkg_resources.get_distribution('mgz').version
        )
        self.session.add(new_file)
        self.session.commit()
        LOGGER.info("[f:%s] add finished, file id: %d, match id: %d", log_id, new_file.id, match.id)
        return True

    def _add_match(self, summary, played, tags, match_hash, series=None, challonge_id=None, voobly_id=None, force=False):
        postgame = summary.get_postgame()
        duration = summary.get_duration()
        from_voobly, ladder = summary.get_ladder()
        settings = summary.get_settings()
        map_name, map_size = summary.get_map()
        map_uuid = None
        completed = summary.get_completed()
        restored, _ = summary.get_restored()
        has_postgame = bool(postgame)
        major_version, minor_version = summary.get_version()
        mod_name, mod_version = summary.get_mod()
        teams = summary.get_teams()
        diplomacy = summary.get_diplomacy()
        log_id = match_hash[:LOG_ID_LENGTH]

        flagged = False
        if restored:
            LOGGER.warning("[m:%s] is restored game", log_id)
            flagged = True

        if not completed:
            LOGGER.warning("[m:%s] was not completed", log_id)
            flagged = True

        if flagged:
            if not force:
                LOGGER.error("[m:%s] skipping add", log_id)
                return False
            LOGGER.warning("[m:%s] adding it anyway", log_id)

        resp = self.aoe2map.get('{}/{}.rms'.format(MAP_URL, map_name)).json()
        if resp['maps']:
            map_uuid = resp['maps'][0]['uuid']

        match = self._get_unique(
            Match, ['hash', 'voobly_id'],
            voobly_id=voobly_id,
            played=played,
            hash=match_hash,
            series=self._get_unique(Series, name=series, challonge_id=challonge_id),
            version=major_version,
            minor_version=minor_version,
            mod_version=mod_version,
            mod=self._get_unique(Mod, name=mod_name),
            voobly=from_voobly,
            voobly_ladder=self._get_unique(VooblyLadder, name=ladder),
            map=self._get_unique(Map, name=map_name, uuid=map_uuid),
            map_size=map_size,
            duration=timedelta(milliseconds=duration),
            completed=completed,
            restored=restored,
            postgame=has_postgame,
            type=settings['type'],
            difficulty=settings['difficulty'],
            population_limit=settings['population_limit'],
            reveal_map=settings['reveal_map'],
            speed=settings['speed'],
            cheats=settings['cheats'],
            lock_teams=settings['lock_teams'],
            diplomacy_type=diplomacy['type'],
            team_size=diplomacy.get('team_size')
        )

        winning_team_id = None
        for data in summary.get_players():
            team_id = None
            for i, team in enumerate(teams):
                if data['number'] in team:
                    team_id = i
            if data['winner']:
                winning_team_id = team_id
            player = self._get_unique(
                Player,
                ['match_id', 'number'],
                civilization=self._get_unique(
                    Civilization,
                    name=mgz.const.CIVILIZATION_NAMES[data['civilization']]
                ),
                team=self._get_unique(
                    Team,
                    ['match', 'team_id'],
                    match=match,
                    team_id=team_id
                ),
                match_id=match.id,
                human=data['human'],
                name=data['name'],
                number=data['number'],
                color_id=data['color_id'],
                start_x=data['position'][0],
                start_y=data['position'][1],
                winner=data['winner'],
                mvp=data['mvp'],
                score=data['score']
            )
            if match.voobly:
                self._guess_match_user(player, data['name'])

        if tags:
            self._add_tags(match, tags)
        match.winning_team_id = winning_team_id
        return match

    def _add_tags(self, match, tags):
        """Add tags to a match."""
        for tag in tags:
            self._get_unique(Tag, name=tag, match=match)

    def _update_match_user(self, match_id, user_data):
        """Update Voobly User info on Match."""
        if user_data:
            player = self.session.query(Player).filter_by(match_id=match_id, color_id=user_data['color_id']).one()
            LOGGER.info("[m:%s] updating voobly user data", player.match.hash[:LOG_ID_LENGTH])
            player.voobly_user_id = user_data['id']
            player.voobly_clan = self._get_unique(VooblyClan, ['id'], id=user_data['clan'])
            player.rate_before = user_data['rate_before']
            player.rate_after = user_data['rate_after']

    def _guess_match_user(self, player, name):
        """Guess Voobly User from a player name."""
        try:
            player.voobly_user = self._get_unique(VooblyUser, ['id'], id=voobly.find_user(self.voobly, name))
            clan = name.split(']')[0][1:] if name.find(']') > 0 else None
            player.voobly_clan = self._get_unique(VooblyClan, ['id'], id=clan)
        except voobly.VooblyError as error:
            LOGGER.warning("failed to lookup Voobly user: %s", error)

    def _get_by_keys(self, table, keys, **kwargs):
        """Get object by unique keys."""
        return self.session.query(table).filter_by(**{k:kwargs[k] for k in keys}).one()

    def _get_unique(self, table, keys=None, **kwargs):
        """Get unique object either by query or creation."""
        if not keys:
            keys = ['name']
        if not any([kwargs[k] is not None for k in keys]):
            return None
        try:
            return self._get_by_keys(table, keys, **kwargs)
        except NoResultFound:
            self.session.begin_nested()
            try:
                obj = table(**kwargs)
                self.session.add(obj)
                self.session.commit()
                return obj
            except IntegrityError:
                self.session.rollback()
                return self._get_by_keys(table, keys, **kwargs)



"""MGZ database schema."""
from datetime import datetime
from sqlalchemy import (
    create_engine, Boolean, DateTime, Column,
    ForeignKey, Integer, Interval, String
)
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import relationship, sessionmaker
from sqlalchemy.schema import ForeignKeyConstraint


BASE = declarative_base()


def get_utc_now():
    """Get current timestamp."""
    return datetime.utcnow()


def get_session(url):
    """Get SQL session."""
    engine = create_engine(url, echo=False)
    session = sessionmaker(bind=engine)()
    BASE.metadata.create_all(engine)
    return session


def reset(url):
    """Reset database - use with caution."""
    engine = create_engine(url, echo=False)
    BASE.metadata.drop_all(engine)


class File(BASE):
    """Represent File."""
    __tablename__ = 'files'
    id = Column(Integer, primary_key=True)
    match_id = Column(Integer, ForeignKey('matches.id'))
    match = relationship('Match', foreign_keys=[match_id])
    hash = Column(String, unique=True, nullable=False)
    filename = Column(String, nullable=False)
    original_filename = Column(String)
    size = Column(Integer, nullable=False)
    owner_number = Column(Integer, nullable=False)
    owner = relationship('Player', primaryjoin='and_(File.match_id==Player.match_id, ' \
                                                    'foreign(File.owner_number)==Player.number)')
    source_id = Column(Integer, ForeignKey('sources.id'))
    source = relationship('Source', foreign_keys=[source_id])
    reference = Column(String)
    added = Column(DateTime, default=get_utc_now)
    parser_version = Column(String, nullable=False)


class Team(BASE):
    """Represent a team."""
    __tablename__ = 'teams'
    team_id = Column(Integer, primary_key=True)
    match_id = Column(Integer, ForeignKey('matches.id'), primary_key=True)
    match = relationship('Match', foreign_keys=match_id)
    members = relationship('Player', primaryjoin='and_(Team.match_id==Player.match_id, ' \
                                                 'Team.team_id==Player.team_id)')


class Match(BASE):
    """Represents Match."""
    __tablename__ = 'matches'
    id = Column(Integer, primary_key=True)
    hash = Column(String, unique=True)
    series_id = Column(Integer, ForeignKey('series.id'))
    series = relationship('Series', foreign_keys=series_id)
    files = relationship('File', foreign_keys='File.match_id', cascade='all, delete-orphan')
    version = Column(String)
    minor_version = Column(String)
    mod_id = Column(Integer, ForeignKey('mods.id'))
    mod = relationship('Mod', foreign_keys=[mod_id])
    mod_version = Column(String)
    voobly = Column(Boolean)
    voobly_ladder_id = Column(Integer, ForeignKey('voobly_ladders.id'))
    voobly_ladder = relationship('VooblyLadder', foreign_keys=[voobly_ladder_id])
    players = relationship('Player', back_populates='match', cascade='all, delete-orphan')
    teams = relationship('Team', foreign_keys='Team.match_id', cascade='all, delete-orphan')
    winning_team_id = Column(Integer)
    winning_team = relationship('Player', primaryjoin='and_(Player.match_id==Match.id, ' \
                                                      'Player.team_id==Match.winning_team_id)')
    losers = relationship('Player', primaryjoin='and_(Player.match_id==Match.id, ' \
                                                 'Player.team_id!=Match.winning_team_id)')
    map_id = Column(Integer, ForeignKey('maps.id'))
    map = relationship('Map', foreign_keys=[map_id])
    map_size = Column(String)
    played = Column(DateTime)
    voobly_id = Column(Integer)
    tags = relationship('Tag', foreign_keys='Tag.match_id', cascade='all, delete-orphan')
    duration = Column(Interval)
    completed = Column(Boolean)
    restored = Column(Boolean)
    postgame = Column(Boolean)
    type = Column(String)
    difficulty = Column(String)
    population_limit = Column(Integer)
    reveal_map = Column(String)
    cheats = Column(Boolean)
    speed = Column(String)
    lock_teams = Column(Boolean)
    diplomacy_type = Column(String, nullable=False)
    team_size = Column(String)


class Player(BASE):
    """Represent Player in a Match."""
    __tablename__ = 'players'
    match_id = Column(Integer, ForeignKey('matches.id'), primary_key=True)
    number = Column(Integer, nullable=False, primary_key=True)
    color_id = Column(Integer, nullable=False)
    voobly_user_id = Column(Integer, ForeignKey('voobly_users.id'))
    voobly_user = relationship('VooblyUser', foreign_keys=voobly_user_id)
    voobly_clan_id = Column(String, ForeignKey('voobly_clans.id'))
    voobly_clan = relationship('VooblyClan', foreign_keys=voobly_clan_id)
    name = Column(String, nullable=False)
    match = relationship('Match', viewonly=True)
    team_id = Column(Integer)
    team = relationship('Team')
    civilization_id = Column(Integer, ForeignKey('civilizations.id'))
    civilization = relationship('Civilization', foreign_keys=[civilization_id])
    start_x = Column(Integer)
    start_y = Column(Integer)
    human = Column(Boolean)
    winner = Column(Boolean)
    mvp = Column(Boolean)
    score = Column(Integer)
    rate_before = Column(Integer)
    rate_after = Column(Integer)
    __table_args__ = (ForeignKeyConstraint(['match_id', 'team_id'], ['teams.match_id', 'teams.team_id']),)


class VooblyUser(BASE):
    """Represents Voobly User."""
    __tablename__ = 'voobly_users'
    id = Column(Integer, primary_key=True)
    matches = relationship('Player', back_populates='voobly_user')


class VooblyClan(BASE):
    """Represents Voobly Clan."""
    __tablename__ = 'voobly_clans'
    id = Column(String, primary_key=True)
    matches = relationship('Player', back_populates='voobly_clan')


class Source(BASE):
    """Represents File Source."""
    __tablename__ = 'sources'
    id = Column(Integer, primary_key=True)
    name = Column(String, nullable=False)


class Mod(BASE):
    """Represents Mod."""
    __tablename__ = 'mods'
    id = Column(Integer, primary_key=True)
    name = Column(String, unique=True)


class Tag(BASE):
    """Tag."""
    __tablename__ = 'tags'
    name = Column(String, primary_key=True)
    match_id = Column(Integer, ForeignKey('matches.id'), primary_key=True)
    match = relationship('Match', foreign_keys=match_id)


class Series(BASE):
    """Represents Series."""
    __tablename__ = 'series'
    id = Column(Integer, primary_key=True)
    name = Column(String)
    challonge_id = Column(String, unique=True)
    matches = relationship('Match', foreign_keys='Match.series_id', cascade='all, delete-orphan')


class VooblyLadder(BASE):
    """Represents Voobly Ladder."""
    __tablename__ = 'voobly_ladders'
    id = Column(Integer, primary_key=True)
    name = Column(String, unique=True, nullable=False)


class Civilization(BASE):
    """Represent Civilization."""
    __tablename__ = 'civilizations'
    id = Column(Integer, primary_key=True)
    name = Column(String, unique=True, nullable=False)


class Map(BASE):
    """Represent Map."""
    __tablename__ = 'maps'
    id = Column(Integer, primary_key=True)
    name = Column(String, unique=True, nullable=False)
    uuid = Column(String, unique=True)



"""Utilities."""
import os
import tempfile
import re
from datetime import datetime
from paramiko import SSHClient
from scp import SCPClient


MGZ_EXT = '.mgz'
ZIP_EXT = '.zip'
CHALLONGE_ID_LENGTH = 9


def copy_file(handle, ssh, path):
    """Copy file to destination store."""
    handle.seek(0)
    with SCPClient(ssh.get_transport()) as scp:
        scp.putfo(handle, path)


def fetch_file(ssh, path):
    """Fetch file from destination store."""
    with tempfile.NamedTemporaryFile() as temp:
        with SCPClient(ssh.get_transport()) as scp:
            scp.get(path, local_path=temp.name)
        temp.flush()
        return open(temp.name, 'rb')


def parse_series_path(path):
    """Parse series name and challonge ID from path."""
    filename = os.path.basename(path)
    start = 0
    challonge_id = None
    challonge_pattern = re.compile('[0-9]+')
    challonge = challonge_pattern.match(filename)
    if challonge:
        challonge_id = int(filename[:challonge.end()])
        start = challonge.end() + 1
    manual_pattern = re.compile(r'.+?\-[0-9]+\-[0-9]+')
    manual = manual_pattern.match(filename)
    if manual:
        challonge_id = filename[manual.start():manual.end()]
        start = manual.end() + 1
    series = filename[start:].replace(ZIP_EXT, '')
    return series, challonge_id


def parse_filename_timestamp(func):
    """Parse timestamp from default rec filename format."""
    if not func.startswith('rec.') or not func.endswith(MGZ_EXT) or len(func) != 23:
        return None
    return datetime(
        year=int(func[4:8]),
        month=int(func[8:10]),
        day=int(func[10:12]),
        hour=int(func[13:15]),
        minute=int(func[15:17]),
        second=int(func[17:19])
    )


def get_store(store_host, port=22, username=None):
    """Get connection to store."""
    ssh = SSHClient()
    ssh.load_system_host_keys()
    parts = store_host.split(':')
    if len(parts) == 2:
        store_host = parts[0]
        port = int(parts[1])
        parts = store_host.split('@')
        if len(parts) == 2:
            username = parts[0]
            store_host = parts[1]
    ssh.connect(store_host, port=port, username=username)
    return ssh
